package main;

import java.awt.Dialog.ModalityType;

import java.util.List;
import java.util.Observable;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;

import view.MainFrame;

import model.databaseModel.*;

/**
 * 
 * Controller-Klasse des MVC-Patterns Einfhrung in den SQL-Dialekt JPQL von JPA
 * siehe hier: http://www.objectdb.com/java/jpa/query
 * 
 * @author Thomas Kramer
 * 
 */
public class Controller extends Observable {
	private MainFrame viewMF = null;

	private Controller controller = null;
	public static EntityManagerFactory emf = null;
	public static EntityManager em = null;

	/**
	 * parameterloser Konstruktor
	 */
	public Controller() {
		controller = this;

		/**
		 * Herstellen der Datenbankverbindung zum Lesen von Daten
		 */
		emf = Persistence.createEntityManagerFactory("JPA");
		em = emf.createEntityManager();

		/**
		 * Erzeuge Hauptfenster und zeige an
		 */
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				viewMF = new MainFrame(controller);
				viewMF.setSize(800, 300);
				viewMF.setVisible(true);
				// positionieren des Fensters: mittig
				viewMF.setLocationRelativeTo(null);

				/**
				 * Beobachter hinzufgen
				 */
				controller.addObserver(viewMF);

				/**
				 * Vorselektion im mainFrame funktioniert nicht, weil Observer
				 * erst NACH Konstruktoraufruf hinzugefgt wird, deswegen
				 * Vorselektion hier.
				 */
				viewMF.ansprechpartnerButton.doClick();

				// viewMF.pack();
				viewMF.setVisible(true);
			}
		});
	}	

	/**
	 * 
	 * Auslesen der Unternehmen-Daten
	 * 
	 */
	public void selectUnternehmen() {
		List<Object> results = null;
		try {
			Query query = em.createQuery("SELECT C FROM Unternehmen C");
			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}

		/**
		 * Views aktualisieren
		 */
		tellChangedData(results);
	}

	/**
	 * 
	 * Auslesen der Unternehmen-Daten und direktes Zurckliefern ber eine Liste
	 * 
	 */
	public static List<Unternehmen> getUnternehmen() {
		List<Unternehmen> results = null;
		try {
			TypedQuery<Unternehmen> query = em.createQuery(
					"SELECT C FROM Unternehmen C", Unternehmen.class);
			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}

		return results;
	}

	/**
	 * 
	 * Auslesen der Ansprechpartner-Daten
	 * 
	 */
	public void selectAnsprechpartner() {
		List<Object> results = null;
		try {
			Query query = em.createQuery("SELECT C FROM Ansprechpartner C");
			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}

		/**
		 * Views aktualisieren
		 */
		tellChangedData(results);
	}

	/**
	 * 
	 * Auslesen der Unternehmen-Ansprechpartner-Zuordnungen
	 * 
	 */
	public void selectZustaendigkeit() {
		List<Object> results = null;
		try {
			Query query = em.createQuery("SELECT C FROM Zustaendigkeit C");
			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}

		/**
		 * Views aktualisieren
		 */
		tellChangedData(results);
	}

	/**
	 * 
	 * Auslesen aller Ansprechpartner zu einem bestimmten Unternehmen
	 * 
	 */
	public void selectZustaendigkeit(Unternehmen unternehmen) {
		List<Object> results = null;
		try {
			Query query = em
					.createQuery("SELECT C FROM Zustaendigkeit C WHERE C.unternehmen=:U_ID");
			query.setParameter("U_ID", unternehmen);

			results = query.getResultList();
		} catch (Exception e) {
			e.printStackTrace(System.out);
		}

		/**
		 * Views aktualisieren
		 */
		tellChangedData(results);
	}

	/**
	 * Einfgen eines neuen Unternehmens
	 * 
	 * @return boolean - ob das Einfgen in DB funktioniert hat oder nicht
	 */
	public boolean insertUnternehmen(String name) {
		boolean result = false;
		if (name.trim().equals("")) {
			return result;
		}

		EntityManagerFactory localEMF = Persistence
				.createEntityManagerFactory("JPA");
		EntityManager localEM = localEMF.createEntityManager();

		EntityTransaction transaction = null;
		try {
			transaction = localEM.getTransaction();
			transaction.begin();

			/**
			 * Neue Datenstze anlegen
			 */
			Unternehmen unternehmen = new Unternehmen();
			unternehmen.setName(name);

			// Datensatz schreiben
			// http://www.developerfusion.com/article/84945/flush-and-clear-or-mapping-antipatterns/
			localEM.persist(unternehmen);

			// (nach schreibenden Zugriffen) alle nderungen werden mit der DB
			// synchronisiert
			// und beim Commit der TX in die DB geschrieben
			// siehe hier:
			// http://www.prozesse-und-systeme.de/jpaEinleitung.html
			localEM.flush();

			// Daten endgltig bermitteln, flush() muss in der Transaktion
			// ablaufen
			// beim commit wird die ID des Datensatzes ermittelt
			transaction.commit();

			result = true;
		} catch (Exception e) {
			if (transaction != null && transaction.isActive()) {
				transaction.rollback();
			}
			result = false;
		} finally {
			localEM.close();
			localEMF.close();
		}
		
		/**
		 * Views aktualisieren
		 */
		selectUnternehmen();
		
		return result;
	}

	/**
	 * Einfgen einer neuen APs mit Unternehmenszuordnung
	 * 
	 * @return boolean - ob das Einfgen in DB funktioniert hat oder nicht
	 */
	public boolean insertAnsprechpartnerAndZustaendigkeit(String name,
			String email, Unternehmen unternehmen) {
		boolean result = false;
		if (name.trim().equals("") || unternehmen == null) {
			return result;
		}

		EntityManagerFactory localEMF = Persistence
				.createEntityManagerFactory("JPA");
		EntityManager localEM = localEMF.createEntityManager();

		EntityTransaction transaction = null;
		try {
			transaction = localEM.getTransaction();
			transaction.begin();

			/**
			 * Neue Datenstze anlegen
			 */
			Ansprechpartner ansprechpartner = new Ansprechpartner();
			ansprechpartner.setName(name);
			ansprechpartner.setEmail(email);

			// Datensatz schreiben
			// http://www.developerfusion.com/article/84945/flush-and-clear-or-mapping-antipatterns/
			localEM.persist(ansprechpartner);

			// (nach schreibenden Zugriffen) alle nderungen werden mit flush()
			// mit
			// der DB synchronisiert
			// und beim Commit der TX in die DB geschrieben
			// siehe hier:
			// http://www.prozesse-und-systeme.de/jpaEinleitung.html
			localEM.flush();

			// Daten endgltig bermitteln, flush() muss in der Transaktion
			// ablaufen
			// beim commit wird die ID des Datensatzes ermittelt
			transaction.commit();

			// Entity aktualisieren, um aktuelle ID zu erhalten
			localEM.refresh(ansprechpartner);

			// //////////////////////////////////////////////////////////////
			// Assoziations-Datensatz schreiben
			// //////////////////////////////////////////////////////////////

			transaction.begin();

			Zustaendigkeit zustaendigkeit = new Zustaendigkeit();
			zustaendigkeit.setAnsprechpartner(ansprechpartner);
			zustaendigkeit.setUnternehmen(unternehmen);

			localEM.persist(zustaendigkeit);
			localEM.flush();

			/**
			 * Daten bermitteln
			 */
			transaction.commit();

			result = true;
		} catch (Exception e) {
			if (transaction != null && transaction.isActive()) {
				transaction.rollback();
			}
			result = false;
		} finally {
			localEM.close();
			localEMF.close();
		}
		
		/**
		 * Views aktualisieren
		 */
		selectAnsprechpartner();
		selectZustaendigkeit();		
		
		return result;
	}
	
	/**
	 * 
	 * Daten weiterreichen, gekapselt in ein ResultContent-Objekt
	 * 
	 * @param results Liste mit eventuellen Ergebnissen
	 */
	private void tellChangedData(List<Object> results)
	{
		if (results != null) {
			if (results.size() > 0) {
				submitData(new ResultContent(results.get(0), results));
			} else {
				submitData(null);				
			}
		} else {
			submitData(null);			
		}		
	}	

	/**
	 * benachrichtige Views, dass Daten gendert wurden.
	 * 
	 * @param object
	 *            gendertes Objekt
	 */
	private void submitData(Object object) {
		// setChanged() markiert das Objekt als sendebereit, sodass bei
		// notifyObservers() Meldungen gegeben werden.
		setChanged();
		notifyObservers(object);
	}
}
